@@ -148,7 +148,7 @@ status_t sstp_stream_connect(sstp_stream_st *client, struct sockaddr *addr,
  * @brief Create the client
  */
 status_t sstp_stream_create(sstp_stream_st **client, event_base_st *base, 
-        SSL_CTX *ssl, const char* name);
+        SSL_CTX *ssl, sstp_option_st *opts);
 
 
 /*!
